@@ -1,16 +1,15 @@
 from .parser import parse
 from .exceptions import SNSinteractionException, SNSserverException
-from .records import get_user_record
+from .records import SNSrecord
 from .objects import SNSinteraction, SNScontent
+from .utils import SNS_PORT, BUFFER_SIZE
 
 import socket
 import re
 
-SNS_PORT = 3735
-
 #TODO: encrypt the whole request with the private key of the user first, to ensure integrity and authenticity. 
 #Since the public key is already known to the server, this does not require any extra steps either.
-def request(type, action, content=None, **headers):
+def request(type, action, content=None, port=SNS_PORT, **headers):
     for key, value in headers.items():
         try:
             headers[key] = str(value)
@@ -19,26 +18,32 @@ def request(type, action, content=None, **headers):
 
     interaction = parse(str(SNSinteraction(type, action, headers, content)))
 
+    if 'From' not in interaction.headers:
+        raise SNSinteractionException(str(interaction), "The interaction is missing From header, causing the interaction not to be sent")
+
+    if interaction.headers['From'].count('@') > 1:
+        raise SNSinteractionException(str(interaction), "The From header contains too many @ characters (max. 1 per address header), causing that the address cannot be determined")
+
+    if 'To' not in interaction.headers:
+        raise SNSinteractionException(str(interaction), "The interaction is missing To header, causing the interaction not to be sent")
+
     if interaction.headers['To'].count('@') > 1:
         raise SNSinteractionException(str(interaction), "The To header contains too many @ characters (max. 1 per address header), causing that the address cannot be determined")
 
-    try:
-        address = interaction.headers['To'].split('@')[1]
-    except:
-        address = get_user_record().provider
+    address = interaction.headers['From'].split('@')[1]
 
     conn = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
     conn.connect((address, SNS_PORT))
+
     conn.sendall(str(interaction).encode('utf-8'))
 
-    buffer_size = 4096
     data = b''
 
     while True:
-        temp_data = conn.recv(buffer_size)
+        temp_data = conn.recv(BUFFER_SIZE)
         data += temp_data
-        if (len(temp_data) < buffer_size): break
-        time.sleep(0.2)
+        if (len(temp_data) < BUFFER_SIZE): break
+        time.sleep(0.1)
 
     conn.close()
 
@@ -56,41 +61,41 @@ def request(type, action, content=None, **headers):
     
     return interactions
 
-def user(action, To, content=None, **headers):
+def user(action, to, from_=None, content=None, **headers):
     encrypted_content = content
 
-    headers['From'] = 'user:' + get_user_record().identifier
-    headers['To'] = To
+    headers['From'] = 'user:' + SNSrecord().this_user.identifier if from_ is None else from_
+    headers['To'] = to
 
     interaction_object = request('USER', action, encrypted_content, **headers)
     
     return interaction_object
 
-def group(action, To, content=None, **headers):
+def group(action, to, from_=None, content=None, **headers):
     encrypted_content = content
 
-    headers['From'] = 'user:' + get_user_record().identifier
-    headers['To'] = To
+    headers['From'] = 'user:' + SNSrecord().this_user.identifier if from_ is None else from_
+    headers['To'] = to
 
     interaction_object = request('GROUP', action, encrypted_content, **headers)
     
     return interaction_object
 
-def post(action, To, content=None, **headers):
+def post(action, to, from_=None, content=None, **headers):
     encrypted_content = content
 
-    headers['From'] = 'user:' + get_user_record().identifier
-    headers['To'] = To
+    headers['From'] = 'user:' + SNSrecord().this_user.identifier if from_ is None else from_
+    headers['To'] = to
 
     interaction_object = request('POST', action, encrypted_content, **headers)
     
     return interaction_object
 
-def reaction(action, To, content=None, **headers):
+def reaction(action, to, from_=None, content=None, **headers):
     encrypted_content = content
 
-    headers['From'] = 'user:' + get_user_record().identifier
-    headers['To'] = To
+    headers['From'] = 'user:' + SNSrecord().this_user.identifier if from_ is None else from_
+    headers['To'] = to
 
     interaction_object = request('REACTION', action, encrypted_content, **headers)
     
